/*
 * Project name:
     RTC3 click (Demonstration of mikroBUS RTC3 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150401:
     - initial release (DM);
 * Description:
      This project is simple demonstration how to read/write date and time from
      BQ32000DR RTC (real-time clock).
      Date and time are read from the RTC every 10 milisecond and printed on LCD.
      Pressing D0 button will write default time and date to RTC module.
      When button is released RTC will start counting.
      User can change default time and date in main function.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
     ext. modules:    RTC3 click board click
                      http://www.mikroe.com/click/RTC3/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Turn on TFT backlight switch SW11.3. (board specific)
     - Place Pull down on PD0 (port D three state switch).
     - RTC3 click board with DS1307 is placed to the mikroBUS socket 1.
       Program uses I2C1 perpheral to comunicate with RTC chip.
 */
 
#include "resources.h"

// Module variables
unsigned char hours, minutes, seconds, day, wday, week, month, year;    // Global date/time variables
char oldstate = 0;

char charge[3] = {0x08, 0x20, 0x45};

// RTC Definitions
#define RTC_ADDR  0x68

/**************************************************************************************************
* Draw Screen
**************************************************************************************************/
void DrawScr(){

  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("RTC3 click  TEST", 75, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMX PRO v7", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

/**************************************************************************************************
* BQ32000DR Functions
**************************************************************************************************/

/**************************************************************************************************
* Read data from RTC BQ32000DR
* input : addres of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read(unsigned char addr){
  unsigned char value;

  I2C1_Start();
  I2C1_Write(RTC_ADDR, addr, 1, END_MODE_RESTART);    // Issue start signal
                                                      // Address BQ32000DR, see BQ32000DR datasheet
                                                      // Start from address
                                                      // Issue repeated start signal
  I2C1_Read(RTC_ADDR, value, 1, END_MODE_STOP);       // Address BQ32000DR for reading R/W=1
                                                      // Read seconds byte
                                                      // Issue stop signal
  return value;
}

/**************************************************************************************************
* Write data from BQ32000DR
* input : addres of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value){
  char buffer[2];
  
  buffer[0] = addr;
  buffer[1] = value;
  
  I2C1_Start();         // Issue start signal
  I2C1_Write(RTC_ADDR, buffer, 2, END_MODE_STOP);   // Start from address
                                                    // Write value to RTC register
                                                    // Issue stop signal
}

/**************************************************************************************************
* Read time from RTC BQ32000DR
* input : pointer to variables where RTC data will be stored
* output: variables with RTC data
**************************************************************************************************/
void Read_Time(unsigned char *p_hours, unsigned char *p_minutes, unsigned char *p_seconds,
               unsigned char *p_day, unsigned char *p_week, unsigned char *p_month, unsigned char *p_year){
  char buffer[11];
  
  I2C1_Start();                                   // Issue start signal
  I2C1_Write(RTC_ADDR, 0, 1, END_MODE_RESTART);   // Address BQ32000DR, see BQ32000DR datasheet
                                                  // Start from address 0
                                                  // Issue repeated start signal
  I2C1_Read(RTC_ADDR, buffer, 11, END_MODE_STOP); // Address BQ32000DR for reading R/W=1
                                                  // Issue stop signal

  *p_seconds = buffer[4];       // Read seconds byte
  *p_minutes = buffer[5];       // Read minutes byte
  *p_hours   = buffer[6];       // Read hours byte
  *p_week    = buffer[7];
  *p_day     = buffer[8];
  *p_month   = buffer[9];
  *p_year    = buffer[10];
}

/**************************************************************************************************
* Write time to RTC BQ32000DR
* input : variables with RTC data
**************************************************************************************************/
void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds,
                unsigned char c_day, unsigned char c_week, unsigned char c_month, unsigned char c_year){
  char buffer[8];
  
  buffer[0] = 0;
  buffer[1] = 0x80;
  buffer[2] = c_minutes;
  buffer[3] = c_hours;
  buffer[4] = c_week;
  buffer[5] = c_day;
  buffer[6] = c_month;
  buffer[7] = c_year;
  
  I2C1_Start();         // Issue start signal
  I2C1_Write(RTC_ADDR, buffer, 8, END_MODE_RESTART);
  
  buffer[0] = 0;
  buffer[1] = 0 | c_seconds;
  I2C1_Write(RTC_ADDR, buffer, 2, END_MODE_STOP);
}

/**************************************************************************************************
* Show on the LCD display
* input : variables with RTC data
**************************************************************************************************/
void Show_Time(){
char loop;
static char oldseconds = 0xFF;
static char oldminutes = 0xFF;
static char oldhours   = 0xFF;
static char oldday     = 0xFF;
static char oldmonth   = 0xFF;
static char old__year  = 0xFF;
static char oldwday    = 0xFF;
  
  char *txt;
  
  seconds  =  ((seconds & 0x70) >> 4)*10 + (seconds & 0x0F);
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);
  hours    =  ((hours & 0x30) >> 4)*10 + (hours & 0x0F);
  wday     =  (wday & 0x07);
  day      =  ((day & 0xF0) >> 4)*10 + (day & 0x0F);
  month    =  ((month & 0x10) >> 4)*10 + (month & 0x0F);
  year     =  ((year & 0xF0)>>4)*10+(year & 0x0F);

  if (wday != oldwday){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    for (loop = 0; loop < 2; loop++){
      switch (oldwday){
        case 1 : {
                  TFT_Write_Text("Sun", 120, 80);
                  break;
                 }
        case 2 : {
                  TFT_Write_Text("Mon", 120, 80);
                  break;
                 }
        case 3 : {
                  TFT_Write_Text("Tue", 120, 80);
                  break;
                 }
        case 4 : {
                  TFT_Write_Text("Wed", 120, 80);
                  break;
                 }
        case 5 : {
                  TFT_Write_Text("Thu", 120, 80);
                  break;
                 }
        case 6 : {
                  TFT_Write_Text("Fri", 120, 80);
                  break;
                 }
        case 7 : {
                  TFT_Write_Text("Sat", 120, 80);
                  break;
                 }
      }
      oldwday = wday;
      TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    }
  }

  if (day != oldday){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldday / 10)   + 48, 100, 120);    // Print tens digit of day variable
    TFT_Write_Char((oldday % 10)   + 48, 115, 120);    // Print oness digit of day variable
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((day / 10)   + 48, 100, 120);    // Print tens digit of day variable
    TFT_Write_Char((day % 10)   + 48, 115, 120);    // Print oness digit of day variable
    oldday = day;
  }
  if (month != oldmonth){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldmonth / 10) + 48, 140, 120);
    TFT_Write_Char((oldmonth % 10) + 48, 155, 120);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((month / 10) + 48, 140, 120);
    TFT_Write_Char((month % 10) + 48, 155, 120);
    oldmonth = month;
  }
  if (year != old__year){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((old__year / 10) + 48, 210, 120);
    TFT_Write_Char((old__year % 10) + 48, 225, 120);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char('2', 180, 120);
    TFT_Write_Char('0', 195, 120);
    TFT_Write_Char((year / 10) + 48, 210, 120);
    TFT_Write_Char((year % 10) + 48, 225, 120);
    old__year = year;
  }

  if (hours != oldhours){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldhours / 10)   + 48, 100, 160);
    TFT_Write_Char((oldhours % 10)   + 48, 115, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((hours / 10)   + 48, 100, 160);
    TFT_Write_Char((hours % 10)   + 48, 115, 160);
    oldhours = hours;
  }
  if (minutes != oldminutes){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldminutes / 10) + 48, 140, 160);
    TFT_Write_Char((oldminutes % 10) + 48, 155, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((minutes / 10) + 48, 140, 160);
    TFT_Write_Char((minutes % 10) + 48, 155, 160);
    oldminutes = minutes;
  }
  if (seconds != oldseconds){
    TFT_Set_Font(Tahoma25x25_Bold, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Char((oldseconds / 10) + 48, 180, 160);
    TFT_Write_Char((oldseconds % 10) + 48, 195, 160);
    TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Char((seconds / 10) + 48, 180, 160);
    TFT_Write_Char((seconds % 10) + 48, 195, 160);
    oldseconds = seconds;
  }
}

/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
void main(){

  TFT_Init_ILI9341_8bit(320,240);
  Delay_ms(500);

  DrawScr();
  // Configure user buttons
  GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_0);
//                                      PD0 is used to write inital data to RTC.

  I2C1_Init();                 // Initialize I2C bus for communication with RTC

  TFT_Set_Font(Tahoma25x25_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Date:", 30, 120); TFT_Write_Char(45, 130, 120); TFT_Write_Char(45, 170, 120);
  TFT_Write_Text("Time:", 30, 160); TFT_Write_Char(58, 130, 160); TFT_Write_Char(58, 170, 160);
  
  // charging super - capacitor
  I2C1_Start();
  I2C1_Write(RTC_ADDR, charge, 3, END_MODE_STOP);

  // Start the test examples
  while(1){                                          // Endless loop
    if (Button(&GPIOD_IDR, 0, 1, 0)) {               // Detect logical one
      oldstate = 1;                                  // Update flag
    }
    if (oldstate && Button(&GPIOD_IDR, 0, 1, 1)) {   // Detect one-to-zero transition
    // Write TIME    16  : 30  : 00  : 1.  Wednesday april  2015
      Write_Time(0x16, 0x30, 0x00, 0x01, 0x04,   0x04, 0x15);
      oldstate = 0;                                  // Update flag
    }

    Read_Time(&hours, &minutes, &seconds, &day, &wday, &month, &year);
    Show_Time();
    Delay_mS(10);
  }
}